var removetabs = {
	_isRTL : null,
	isRTL : function() {
		if(removetabs._isRTL == null){
			var chromedir;
			try{
				var pref = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefBranch);
				var locale = pref.getCharPref("general.useragent.locale");
				chromedir = pref.getCharPref("intl.uidirection." + locale);
			}catch(e){}
			removetabs._isRTL = (chromedir == "rtl");
		}
		return (removetabs._isRTL);
	},

	removeLeft : function(usecontext) {
		var ctab = (usecontext ? gBrowser.mContextTab : gBrowser.selectedTab);
		if(removetabs.isRTL()){
			removetabs.removeAfter(ctab);
		}else{
			removetabs.removeBefore(ctab);
		}
	},
	
	removeRight : function(usecontext) {
		var ctab = (usecontext ? gBrowser.mContextTab : gBrowser.selectedTab);
		if(removetabs.isRTL()){
			removetabs.removeBefore(ctab);
		}else{
			removetabs.removeAfter(ctab);
		}
	},
	
	removeBefore : function(ctab) {
		var tabs = gBrowser.tabContainer.childNodes;
		var i;
		for(i=tabs.length-1; tabs[i] != ctab; i--){}
		for(i--; i>=0; i--){
			gBrowser.removeTab(tabs[i]);
		}
		removetabs.updateEnabled();
	},
	
	removeAfter : function(ctab) {
		var tabs = gBrowser.tabContainer.childNodes;
		for(var i=tabs.length-1; tabs[i] != ctab; i--){
			gBrowser.removeTab(tabs[i]);
		}
		removetabs.updateEnabled();
	},
	
	removeOther : function(usecontext) {
		var ctab = (usecontext ? gBrowser.mContextTab : gBrowser.selectedTab);
		gBrowser.removeAllTabsBut(ctab);
		removetabs.updateEnabled();
	},
	
	removeAll : function(usecontext) {
		var ctab = gBrowser.addTab(removetabs.getRemoveAllPage());
		gBrowser.removeAllTabsBut(ctab);
		if(gBrowser.tabContainer.childNodes.length > 1) {
			gBrowser.removeTab(ctab);
		}
		removetabs.updateEnabled();
	},
	
	getRemoveAllPage : function() {
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
		var prefBranch = prefSvc.getBranch("extensions.removetabs.");
	
		switch(prefBranch.getIntPref("removeall_type")) {
		case 0:
			return "about:blank";
		case 1:
			return gBrowser.homePage;
		case 2:
			return prefBranch.getCharPref("removeall_url");
		default:
			return "about:blank";
		}
	},
	
	setDisabled : function(id, value) {
		var item = gBrowser.ownerDocument.getElementById(id);
		if(item == null){ return; }
		item.setAttribute("disabled", value);
	},
	
	updateEnabled : function() {
		var tabs = gBrowser.tabContainer.childNodes;
		if(removetabs.isRTL()){
			removetabs.updateEnabled_rtl(tabs);
		}else{
			removetabs.updateEnabled_ltr(tabs);
		}
		removetabs.setDisabled("tab-removeallmenu",
			(tabs.length <= 1 && 
			 (gBrowser.currentURI.spec == "" || gBrowser.currentURI.spec == "about:blank")));
		removetabs.setDisabled("removeotherbutton", (tabs.length <= 1));
		removetabs.setDisabled("removeallbutton",
			(tabs.length <= 1 && 
			 (gBrowser.currentURI.spec == "" || gBrowser.currentURI.spec == "about:blank")));
	},
	
	updateEnabled_ltr : function(tabs) {
		removetabs.setDisabled("tab-removeleftmenu",  (tabs[0] == gBrowser.mContextTab));
		removetabs.setDisabled("tab-removerightmenu", (tabs[tabs.length-1] == gBrowser.mContextTab));
		removetabs.setDisabled("removeleftbutton",    (tabs[0] == gBrowser.selectedTab));
		removetabs.setDisabled("removerightbutton",   (tabs[tabs.length-1] == gBrowser.selectedTab));
	},
	
	updateEnabled_rtl : function(tabs) {
		removetabs.setDisabled("tab-removeleftmenu",  (tabs[tabs.length-1] == gBrowser.mContextTab));
		removetabs.setDisabled("tab-removerightmenu", (tabs[0] == gBrowser.mContextTab));
		removetabs.setDisabled("removeleftbutton",    (tabs[tabs.length-1] == gBrowser.selectedTab));
		removetabs.setDisabled("removerightbutton",   (tabs[0] == gBrowser.selectedTab));
	},
	
	addMenuItem : function(browser, menuid) {
		if(!browser){ return; }
		var removetabsMenu = browser.ownerDocument.getElementById(menuid).cloneNode(false);
		if(!removetabsMenu){ return; }
		removetabsMenu.id = "tab-" + menuid;
		var menuParent = removetabs.getTabContextMenu(browser);
		var pos = 4;
		if(pos >= menuParent.childNodes.length){
		  menuParent.appendChild(removetabsMenu);
		}else{
		  menuParent.insertBefore(removetabsMenu, menuParent.childNodes[pos]);
		}
	},
	
	addMenuItems : function(browser) {
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
		var prefBranch = prefSvc.getBranch("extensions.removetabs.");
	
		if(prefBranch.getBoolPref("add_removeleft_menu")) {
			removetabs.addMenuItem(browser, "removeleftmenu");
		}
		if(prefBranch.getBoolPref("add_removeright_menu")) {
			removetabs.addMenuItem(browser, "removerightmenu");
		}
		if(prefBranch.getBoolPref("add_removeall_menu")) {
			removetabs.addMenuItem(browser, "removeallmenu");
		}
	},
	
	init : function() {
		removetabs.addMenuItems(gBrowser);
	
		var menuParent = removetabs.getTabContextMenu(gBrowser);
		menuParent.addEventListener("popupshowing", removetabs.updateEnabled, false);
		var contentArea = gBrowser.ownerDocument.getElementById("appcontent");
		contentArea.addEventListener("load", removetabs.updateEnabled, true);
		contentArea.addEventListener("select", removetabs.updateEnabled, false);
		contentArea.addEventListener("unload", removetabs.updateEnabled, true);
		removetabs.updateEnabled();
	},
	
	getTabContextMenu : function(browser) {
		var doc = browser.ownerDocument;
		var tabbrowserTabs = doc.getElementById("tabbrowser-tabs");
		if(tabbrowserTabs == null){
			// for Firefox 3.6
			tabbrowserTabs = browser;
		}
		return doc.getAnonymousElementByAttribute(tabbrowserTabs, "anonid", "tabContextMenu");
	}
};
